searchNodes=[{"doc":"Para is an Elixir library that provides structured and declarative way to parse and validate parameters. Para uses Ecto under the hood and therefore inherits most of its utilities such as changeset and built-in validators. Usage Let's imagine that you have a controller named Web.UserController and wanted to validate the parameters for its :create and :update actions First, let's define your parameters schema defmodule Web.UserPara do use Para validator :create do required :name , :string required :age , :integer required :email , :string optional :phone , :string end validator :update do required :name , :string required :age , :integer required :email , :string optional :phone , :string end end You can now use this module as a validator in your controller defmodule Web.UserController do use Web , :schema alias Web.UserPara def create ( conn , params ) do with { :ok , data } &lt;- UserPara . validate ( :create , params ) do # ... end end end Inline validators Inline validator is a convenient way to validate your fields. This is especially useful when you need to perform some basic validation using Ecto.Changeset 's built-in validators. defmodule UserPara do use Para validator :update do required :name , :string , validator : { :validate_length , [ min : 3 , max : 100 ] } end end You can also use custom inline validators by supplying the function name as an atom. Custom inline validator will receive changeset and the original params as the arguments. defmodule UserPara do use Para validator :update do required :age , :string , validator : :validate_age end def validate_age ( changeset , params ) do # ... end end Callback validator Sometimes, you might want to use custom validators or need to perform additional data manipulations. For this, you can use the callback/1 macro. The callback/1 macro will always be the last function to be called after the validator has parsed and validated the parameters. defmodule Web.UserPara do use Para validator :create do required :name , :string required :age , :integer required :email , :string optional :phone , :string callback :create_validators end def create_validators ( changeset , params ) do changeset |&gt; format_email ( params ) |&gt; format_phone ( params ) |&gt; validate_age ( ) end def format_email ( changeset , params ) do # ... end def format_phone ( changeset , params ) do # ... end def validate_age ( changeset ) do # ... end end","ref":"Para.html","title":"Para","type":"module"},{"doc":"Define custom callback to perform any additional parsing or validation to the processed changeset or parameters","ref":"Para.html#callback/1","title":"Para.callback/1","type":"macro"},{"doc":"Define an embedded array of maps field","ref":"Para.html#embeds_many/2","title":"Para.embeds_many/2","type":"macro"},{"doc":"Define an embedded map field","ref":"Para.html#embeds_one/2","title":"Para.embeds_one/2","type":"macro"},{"doc":"Define an optional field. Similar to required/3 , it also accepts the same Options","ref":"Para.html#optional/3","title":"Para.optional/3","type":"macro"},{"doc":"Define a required field. Options :default - Assign a default value if the not set by input parameters :validator - Define either one of the built-in Ecto.Changeset's validators or use your own custom inline validator. Refer: Custom inline validator :droppable - Drop the field when the key doesn't exist in parameters. This is useful when you need to perform partial update by leaving out certain fields. Custom inline validator You can define your own validator as such: def validate_country ( changeset , field ) do # ... end Then use it as an inline validator for your field validator :create do required :country , :string , [ validator : :validate_country ] end","ref":"Para.html#required/3","title":"Para.required/3","type":"macro"},{"doc":"Define a validator schema with an action name and field definitions. This will generate a new function called validate/2 with the action name and params as the arguments. iex&gt; defmodule UserPara do ...&gt; use Para ...&gt; ...&gt; validator :create do ...&gt; required :name ...&gt; end ...&gt; end ...&gt; ...&gt; UserPara . validate ( :create , %{ &quot;name&quot; =&gt; &quot;Syamil MJ&quot; } ) { :ok , %{ name : &quot;Syamil MJ&quot; } }","ref":"Para.html#validator/2","title":"Para.validator/2","type":"macro"},{"doc":"","ref":"Para.html#t:t/0","title":"Para.t/0","type":"type"},{"doc":"CHANGELOG","ref":"changelog.html","title":"CHANGELOG","type":"extras"},{"doc":"Improvements Simplify spec defaults Refactor function organization Fixes embeds callback","ref":"changelog.html#v0-2-1","title":"CHANGELOG - [v0.2.1]","type":"extras"},{"doc":"Added embeds_one - support embedding a single map embeds_many - support embedding list of maps Note - You can always use {:map, :string} or {:array, :map} as field type if you don't care about validation for the embedded fields. Changed Use converted params to ensure consistency of map keys","ref":"changelog.html#v0-2-0","title":"CHANGELOG - [v0.2.0]","type":"extras"},{"doc":"Added Implement validator callback","ref":"changelog.html#v0-1-1","title":"CHANGELOG - [v0.1.1]","type":"extras"},{"doc":"Added all the things","ref":"changelog.html#v0-1-0","title":"CHANGELOG - [v0.1.0]","type":"extras"}]